#pragma once

#include <atlapp.h>
#include <atlgdi.h>
#include <GdiPlus.h>
#include <GdiPlusTypes.h>

using namespace Gdiplus;

class CGDIPlusDC : public WTL::CDCHandle
{
public:
	CGDIPlusDC(HDC hDC=NULL):WTL::CDCHandle(hDC)
	{
	}
	virtual ~CGDIPlusDC()
	{
	}
	void MeasureString(LPCTSTR lpchText, LPCTSTR pszcFontName, int nFontSize, BOOL bBold, BOOL bItalic, RectF& rectOut)
	{
		ATLASSERT(m_hDC != NULL);

		int nFontStyle = Gdiplus::FontStyleRegular;
		if (bBold)
		{
			nFontStyle += Gdiplus::FontStyleBold;
		}
		if (bItalic)
		{
			nFontStyle += Gdiplus::FontStyleItalic;
		}
		Gdiplus::FontStyle fontStyle = static_cast<Gdiplus::FontStyle>(nFontStyle);

		Gdiplus::Graphics g(m_hDC);
		RectF layoutRect(0, 0, rectOut.Width, rectOut.Height);
		USES_CONVERSION;
		Gdiplus::Font font(CT2CW(pszcFontName), REAL(nFontSize), fontStyle);
		g.MeasureString(CT2CW(lpchText), _tcslen(lpchText), &font, layoutRect, &rectOut);		
	}
	void FillSolidRect(int x, int y, int cx, int cy, COLORREF clr)
	{
		ATLASSERT(m_hDC != NULL);

		RECT rect = { x, y, x + cx, y + cy };
		FillSolidRect(&rect, clr);
	}
	void FillSolidRect(LPCRECT lpRect, COLORREF clr)
	{
		ATLASSERT(m_hDC != NULL);

		Gdiplus::Graphics g(m_hDC);
		Gdiplus::SolidBrush brush(Gdiplus::Color(0xff, GetRValue(clr), GetGValue(clr), GetBValue(clr)));
		g.FillRectangle(&brush, lpRect->left, lpRect->top, lpRect->right-lpRect->left, lpRect->bottom-lpRect->top);
	}
	void Draw3dRect(LPCRECT lpRect, COLORREF clrTopLeft, COLORREF clrBottomRight)
	{
		Draw3dRect(lpRect->left, lpRect->top, lpRect->right - lpRect->left,
			lpRect->bottom - lpRect->top, clrTopLeft, clrBottomRight);
	}

	void Draw3dRect(int x, int y, int cx, int cy, COLORREF clrTopLeft, COLORREF clrBottomRight)
	{
		FillSolidRect(x, y, cx - 1, 1, clrTopLeft);
		FillSolidRect(x, y, 1, cy - 1, clrTopLeft);
		FillSolidRect(x + cx - 1 , y, 1, cy, clrBottomRight);
		FillSolidRect(x, y + cy - 1, cx, 1, clrBottomRight);
	}
	void DrawText(LPCTSTR lpchText, int cchText, LPCRECT lprc, int nType,
				LPCTSTR pszcFontName, int nFontSize, BOOL bBold, BOOL bItalic, BOOL bUnderline, COLORREF clrFont)
	{
		ATLASSERT(m_hDC != NULL);
		Gdiplus::Graphics g(m_hDC);
		
		USES_CONVERSION;

		int nFontStyle = Gdiplus::FontStyleRegular;
		if (bBold)
		{
			nFontStyle += Gdiplus::FontStyleBold;
		}
		if (bItalic)
		{
			nFontStyle += Gdiplus::FontStyleItalic;
		}
		if ( bUnderline )
		{
			nFontStyle += Gdiplus::FontStyleUnderline;
		}
		Gdiplus::FontStyle fontStyle = static_cast<Gdiplus::FontStyle>(nFontStyle);
		Gdiplus::StringFormat format;
		//format.SetAlignment(Gdiplus::StringAlignmentNear);
		//format.SetFormatFlags(Gdiplus::StringFormatFlagsDisplayFormatControl);

		CRect rcPosition = lprc;
		if (nType & DT_CENTER)
		{
			format.SetAlignment(Gdiplus::StringAlignmentCenter);
		}
		else if (nType & DT_RIGHT)
		{
			format.SetAlignment(Gdiplus::StringAlignmentFar);			
		}
		
		if (nType & DT_VCENTER)
		{
			format.SetLineAlignment(Gdiplus::StringAlignmentCenter);
			rcPosition.top += rcPosition.Height()/2;
		}
		else if (nType & DT_BOTTOM)
		{
			format.SetLineAlignment(Gdiplus::StringAlignmentFar);
			rcPosition.top += rcPosition.Height();
		}

		if( nType & DT_END_ELLIPSIS )
		{
			format.SetTrimming( Gdiplus::StringTrimmingEllipsisCharacter );
		}

		Gdiplus::RectF origin(static_cast<FLOAT>(rcPosition.left), static_cast<FLOAT>(rcPosition.top),
							static_cast<FLOAT>(rcPosition.Width()), static_cast<FLOAT>(rcPosition.Height()));
		Gdiplus::Font font(CT2CW(pszcFontName), REAL(nFontSize), fontStyle);
		Gdiplus::SolidBrush brush(Gdiplus::Color(0xf0, GetRValue(clrFont), GetGValue(clrFont), GetBValue(clrFont)));
		
		g.DrawString(CT2CW(lpchText), cchText, &font, origin, &format, &brush);		
	}
private: 
};